/*
 * Copyright (c) 1999 by Sun Microsystems, Inc.
 * All rights reserved.
 */

#ifndef _SYS_FIBRE_CHANNEL_FCA_USOCVAR_H
#define	_SYS_FIBRE_CHANNEL_FCA_USOCVAR_H

#pragma ident	"@(#)usocvar.h	1.1	99/08/02 SMI"

#ifdef __cplusplus
extern "C" {
#endif

/*
 * usocvar.h - SOC+ Driver data struct definitions
 */

/*
 * Define default name and # of SOC+s to allocate to the system
 */

#define	USOC_PORTA_NAME		"0"	/* node for port a */
#define	USOC_PORTB_NAME		"1"	/* node for port b */
#define	USOC_NT_PORT		NULL
#define	USOC_INIT_ITEMS		5
#define	USOC_WWN_SIZE		8

/*
 * Defines for the Circular Queues
 */
#define	USOC_MAX_CQ_ENTRIES	256	/* Maximum number of CQ entries. */
#define	USOC_CQ_SIZE 		(sizeof (cqe_t) * USOC_MAX_CQ_ENTRIES)

#define	USOC_SMALL_CQ_ENTRIES	8	/* Number of CQ entries for a small Q */

#define	USOC_N_CQS		4	/* Number of queues we use */
#define	USOC_HW_N_CQS		4	/* Number of queues the hardware has */
#define	USOC_CQ_ALIGN		64	/* alignment boundary */

/*
 * Misc. Macros
 */
#define	USOC_POOL_SIZE		2112
#define	USOC_SVC_LENGTH		80

#define	FABRIC_FLAG		1
#define	NPORT_FLAG		2

#define	USOC_STATUS_DIAG_BUSY		82565
#define	USOC_STATUS_DIAG_INVALID	82566
#define	FCIO_DIAG_LBTFQ		(FIOC|203)
#define	USOC_DIAG_LBTFQ		0x0a
#define	PORT_LBF_PENDING	0x00100000
#define	USOC_LBF_TIMEOUT	15		/* seconds */

/* Macros to speed handling of 32-bit IDs */
#ifdef _LP64
#define	USOC_ID_GET(x, y)	(((y) == KM_SLEEP) ?\
	(uint32_t)id32_alloc(x) : (uint32_t)id32_alloc_nosleep(x))
#define	USOC_ID_LOOKUP(x)	id32_lookup((x))
#define	USOC_ID_FREE(x)		id32_free((x))
#else
#define	USOC_ID_GET(x, y)	((uint32_t)(x))
#define	USOC_ID_LOOKUP(x)	(x)
#define	USOC_ID_FREE(x)
#endif

#define	USOC_QFULL		82565
#define	USOC_UNAVAIL		82566

typedef	struct flb_hdr {
	uint_t max_length;
	uint_t length;
} flb_hdr_t;

/*
 * USOC UNIX circular queue descriptor.
 */

typedef struct usoc_kernel_cq {
	kmutex_t	skc_mtx;	/* MT lock for CQ manipulation  */
	kcondvar_t	skc_cv;		/* cond var for CQ manipulation. */
	ddi_dma_handle_t skc_dhandle;	/* DDI DMA handle to CQ. */
	ddi_dma_cookie_t skc_dcookie;	/* DDI DMA Cookie. */
	ddi_acc_handle_t skc_acchandle;	/* DDI DMA access handle */
	usoc_cq_t	*skc_xram_cqdesc; /* Pointer to XRAM CQ desc */
	caddr_t		skc_cq_raw;	/* Pointer to unaligned CQ mem pool */
	cqe_t		*skc_cq;	/* Pointer to CQ memory pool. */
	uchar_t		skc_in;		/* Current Input pointer. */
	uchar_t		skc_out;	/* Current Input pointer. */
	uchar_t		skc_last_index;	/* Last cq index. */
	uchar_t		skc_seqno;	/* Current Go Around in CQ. */
	uchar_t		skc_full;	/* Indication of full. */
	uchar_t		skc_saved_out;	/* Current Input pointer. */
	uchar_t		skc_saved_seqno;	/* Current Go Around in CQ. */
	timeout_id_t	deferred_intr_timeoutid;
	struct usoc_pkt_priv	*skc_overflowh;		/* cq overflow head */
	struct usoc_pkt_priv	*skc_overflowt;		/* cq overflow tail */
} usoc_kcq_t;

/*
 * Values for skc_full
 */
#define	USOC_SKC_FULL	1
#define	USOC_SKC_SLEEP	2

/*
 * State change callback routine descriptor
 *
 * There is one entry in this list for each hba that is attached
 * to this port.
 * This structure will need to be mutex protected when parallel
 * attaches are supported.
 */
typedef struct usoc_unsol_cb {
	struct usoc_unsol_cb	*next;
	uchar_t			type;
	void			(*statec_cb)(void *, uint32_t);
	void			(*els_cb)(void *, cqe_t *, caddr_t);
	void			(*data_cb)(void *, cqe_t *, caddr_t);
	void			*arg;
} usoc_unsol_cb_t;

/*
 * Unsolicited buffer descriptors
 */
typedef struct usoc_unsol_buf {
	uint32_t		pool_type;	/* FC-4 type */
	uint32_t		pool_id; /* pool id for this buffer pool */
	uint32_t		pool_buf_size;	/* buf size of each pool buf */
	uint16_t		pool_nentries;	/* no.of bufs in pool */
	uint32_t		pool_hdr_mask;  /* header mask for pool */
	fc_unsol_buf_t		*pool_fc_ubufs; /* array of unsol buf structs */
	struct usoc_unsol_buf	*pool_next;
	struct pool_dma_res {
		ddi_dma_handle_t	pool_dhandle;
		ddi_dma_cookie_t	pool_dcookie;
		ddi_acc_handle_t	pool_acchandle;
		caddr_t			pool_buf;
	} *pool_dma_res_ptr;	/* dma resources for buffers in the pool */
} usoc_unsol_buf_t;

/*
 * SOC+ port status decriptor.
 */
typedef struct usoc_port {
	uint32_t		sp_status;	/* port status */
	struct usoc_state	*sp_board;	/* hardware for instance */

	uint32_t		sp_src_id;	/* Our nport id */
	uint32_t		sp_port;	/* Our physical port (0, 1) */
	la_wwn_t		sp_p_wwn;	/* Our Port WorldWide Name */

	void			(*sp_statec_callb)();
	void			(*sp_unsol_callb)();
	opaque_t		sp_tran_handle;
	uint32_t		sp_open;	/* open count */
	int			sp_pktinits;
	usoc_unsol_buf_t	*usoc_unsol_buf;

	kmutex_t		sp_mtx;		/* Per port mutex */
	kcondvar_t		sp_cv;		/* Per port condvariable */
	struct usoc_pkt_priv	*sp_flush_head;
	uint32_t		sp_of_ncmds;

	kcondvar_t		sp_unsol_cv;	/* XXX: temp for unsol ioctl */
	fc_unsol_buf_t		*sp_unsol_buf;	/* XXX: pointer to rcvd ub */
	kmutex_t		sp_unsol_mtx;	/* XXX: to serialize ioctls */
} usoc_port_t;

#define	PORT_FABRIC_PRESENT	0x00000001
#define	PORT_OFFLINE		0x00000002
#define	NPORT_LOGIN_SUCCESS	0x00000004
#define	PORT_LOGIN_ACTIVE	0x00000008
#define	PORT_LOGIN_RECOVERY	0x00000010
#define	PORT_ONLINE_LOOP	0x00000020
#define	PORT_ONLINE		0x00000040
#define	PORT_STATUS_FLAG	0x00000080
#define	PORT_STATUS_MASK	0x000000ff
#define	PORT_OPEN		0x00000100
#define	PORT_CHILD_INIT		0x00000200
#define	PORT_BOUND		0x00000400
#define	PORT_LILP_PENDING	0x00001000
#define	PORT_LIP_PENDING	0x00002000
#define	PORT_ABORT_PENDING	0x00004000
#define	PORT_ELS_PENDING	0x00008000
#define	PORT_BYPASS_PENDING	0x00010000
#define	PORT_OFFLINE_PENDING	0x00020000
#define	PORT_ADISC_PENDING	0x00040000
#define	PORT_RLS_PENDING	0x00080000
#define	PORT_FLOGI_PENDING	0x00100000
#define	PORT_NS_PENDING		0x00200000
#define	PORT_IN_LINK_RESET	0x00400000
#define	PORT_OUTBOUND_PENDING	0x00800000
#define	PORT_DIAG_PENDING	0x01000000

#define	USOC_NOINTR_POLL_DELAY_TIME	1000    /* usec */

#define	USOC_LILP_TIMEOUT		30
#define	USOC_LIP_TIMEOUT		30
#define	USOC_ABORT_TIMEOUT		20		/* sec */
#define	USOC_BYPASS_TIMEOUT		5
#define	USOC_OFFLINE_TIMEOUT		90
#define	USOC_ADISC_TIMEOUT		15
#define	USOC_RLS_TIMEOUT		15
#define	USOC_DIAG_TIMEOUT		15
#define	USOC_PKT_TIMEOUT		45

#define	USOC_MIN_PACKETS		10	/* Advisory; Don't worry */
#define	USOC_MAX_PACKETS		1024	/* Advisory; Don't worry */
#define	USOC_NUM_TIMEOUT_THREADS	4	/* 2 threads per port */
#define	USOC_WATCH_TIMER		1000000	/* usec */

/*
 * usoc_timetag_t is the timeout record for timing out requests
 */
typedef struct usoc_timetag {
	void			(*sto_func) (fc_packet_t *);
	fc_packet_t		*sto_pkt;
	uint32_t		sto_ticks;
	struct usoc_timetag	*sto_next;
	struct usoc_timetag	*sto_prev;
	struct usoc_timetag	*sto_tonext;	/* timeout list */
	uint32_t		sto_flags;
} usoc_timetag_t;

#define	USOCTIME_DORMANT	0x00
#define	USOCTIME_INIT		0x01
#define	USOCTIME_TICKING	0x02

/*
 * We wait for up to USOC_INITIAL_ONLINE seconds for the first
 * usoc to come on line. The timeout in the usoc firmware is 10 seconds.
 * The timeout is to let any outstanding commands drain before
 * coming back on line, after going off-line.
 */
#define	USOC_INITIAL_ONLINE	30	/* secs for first online from usoc */

/*
 * USOC state structure
 */

typedef struct usoc_state {
	dev_info_t		*usoc_dip;
	caddr_t 		usoc_eeprom;	/* pointer to soc+ eeprom */
	caddr_t 		usoc_xrp;	/* pointer to soc+ xram */
	usoc_reg_t		*usoc_rp;	/* pointer to soc+ registers */

	usoc_cq_t		*usoc_xram_reqp; /* addr of request queue */
	usoc_cq_t		*usoc_xram_rspp; /* addr of response queue */

	usoc_kcq_t		usoc_request[USOC_N_CQS]; /* request queues */
	usoc_kcq_t		usoc_response[USOC_N_CQS]; /* response queues */

	uint32_t		usoc_busy;	/* busy flag */
	uint32_t		usoc_shutdown;
	uint32_t		usoc_cfg;	/* copy of the config reg */

	kmutex_t		usoc_k_imr_mtx;	/* mutex for interrupt masks */
	uint32_t		usoc_k_imr;	/* copy of soc+'s mask reg */


	kmutex_t		usoc_board_mtx;	/* Per board mutex */
	kcondvar_t		usoc_board_cv;	/* Per board cond variable */

	ddi_iblock_cookie_t	usoc_iblkc;	/* interrupt cookies */
	ddi_idevice_cookie_t	usoc_idevc;

	uchar_t			*usoc_pool;	/* unsolicited buffer pool */
	ddi_dma_handle_t	usoc_pool_dhandle;
	ddi_dma_cookie_t	usoc_pool_dcookie;
	ddi_acc_handle_t	usoc_pool_acchandle;

	kmutex_t		usoc_time_mtx;

	usoc_timetag_t		*usoc_timelist;

	taskq_t			*usoc_task_handle;
	timeout_id_t		usoc_watch_tid;

				/* handles to soc+ ports */
	usoc_port_t		usoc_port_state[N_USOC_NPORTS];
	la_wwn_t		usoc_n_wwn;	/* Our Node WorldWide Name */
	char			usoc_service_params[USOC_SVC_LENGTH];

	char			usoc_name[MAXPATHLEN];
	kstat_t			*usoc_ksp;
	struct usoc_stats	usoc_stats;	/* kstats */
	uint32_t		usoc_ncmds;
	uint32_t		usoc_throttle;
	uint32_t		usoc_ticker;
} usoc_state_t;

/*
 * Structure used when the usoc driver needs to issue commands of its own
 */

typedef struct usoc_priv_cmd {
	void			*fapktp;
	uint32_t		flags;
	caddr_t			cmd;
	caddr_t			rsp;
	ddi_dma_handle_t	cmd_handle;
	ddi_acc_handle_t	cmd_acchandle;
	ddi_dma_handle_t	rsp_handle;
	ddi_acc_handle_t	rsp_acchandle;
	void 			(*callback)();	/* callback to ULP, if any */
	void			*arg;		/* callback arg */
	caddr_t			*payload;	/* payload callback or stash */
} usoc_priv_cmd_t;

#define	PACKET_VALID		0x80000000
#define	PACKET_DORMANT		0x00000001
#define	PACKET_IN_TRANSPORT	0x00000002
#define	PACKET_IN_ABORT		0x00000004
#define	PACKET_IN_PROCESS	(PACKET_IN_TRANSPORT | PACKET_IN_ABORT)
#define	PACKET_INTERNAL_ABORT	0x00000008
#define	PACKET_INTERNAL_PACKET	0x40000000
#define	PACKET_IN_TIMEOUT	0x00100000
#define	PACKET_IN_INTR		0x00200000
#define	PACKET_NO_CALLBACK	0x00400000
#define	PACKET_IN_OVERFLOW_Q	0x00800000
#define	PACKET_RETURNED		0x02000000

typedef struct usoc_pkt_priv {
	usoc_port_t		*spp_portp;		/* our port */
	uint32_t		spp_flags;	/* private packet flags */
	void			(*spp_saved_comp) (fc_packet_t *);
	kmutex_t		spp_mtx;
	usoc_request_t		spp_sr;
	uint32_t		spp_endflag;
	uint32_t		spp_diagcode;
	uint32_t		spp_token;
	usoc_timetag_t		spp_timetag;
	fc_packet_t		*spp_packet;
	struct usoc_pkt_priv	*spp_next;
} usoc_pkt_priv_t;

#define	UB_VALID		0x80000000
#define	UB_IN_FCA		0x00000001
#define	UB_TEMPORARY		0x00000002

typedef struct usoc_ub_priv {
	usoc_port_t		*ubp_portp;
	uint32_t 		ubp_flags;
	struct pool_dma_res	*ubp_pool_dma_res;
	uint64_t		ubp_ub_token;
} usoc_ub_priv_t;

#ifdef __cplusplus
}
#endif

#endif /* !_SYS_FIBRE_CHANNEL_FCA_USOCVAR_H */
